@@ -27,7 +27,6 @@ import net.sf.saxon.resource.CollectionURIResolverWrapper;
 import net.sf.saxon.trace.TraceEventMulticaster;
 import net.sf.saxon.trans.*;
 import net.sf.saxon.trans.rules.RuleManager;
-import net.sf.saxon.tree.tiny.Statistics;
 import net.sf.saxon.tree.tiny.TinyBuilder;
 import net.sf.saxon.tree.wrapper.SpaceStrippedDocument;
 import net.sf.saxon.tree.wrapper.SpaceStrippedNode;
@@ -606,6 +605,16 @@ public class Controller implements ContextOriginator {
         this.globalContextItemPreset = true;
     }
 
+    /**
+     * Reset the global context item to null. This clears any previous setting of the global context
+     * item.
+     */
+
+    public void clearGlobalContextItem() {
+        this.globalContextItem = null;
+        this.globalContextItemPreset = false;
+    }
+
 
     /**
      * Get the item used as the context for evaluating global variables. In XQuery this
@@ -1136,7 +1145,7 @@ public class Controller implements ContextOriginator {
      * users (it is done automatically when transform() is invoked). However, it is available as a low-level API
      * especially for use with XQuery.
      *
-     * @param params the values of stylesheet parameters. Changed in 9.9.0.2 so this no longer includes
+     * @param params the values of stylesheet parameters. Changed in 9.9.1.1 so this no longer includes
      *               static parameters (which are already available in the {@link PreparedStylesheet}).
      * @throws XPathException if an error occurs, for example if a required parameter is not supplied.
      */
@@ -1311,7 +1320,7 @@ public class Controller implements ContextOriginator {
         Builder sourceBuilder = makeBuilder();
         sourceBuilder.setUseEventLocation(true);
         if (sourceBuilder instanceof TinyBuilder) {
-            ((TinyBuilder) sourceBuilder).setStatistics(Statistics.SOURCE_DOCUMENT_STATISTICS);
+            ((TinyBuilder) sourceBuilder).setStatistics(config.getTreeStatistics().SOURCE_DOCUMENT_STATISTICS);
         }
         Receiver r = sourceBuilder;
         SpaceStrippingRule spaceStrippingRule = NoElementsSpaceStrippingRule.getInstance();
