@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,13 +9,11 @@ package net.sf.saxon.expr;
 
 import net.sf.saxon.expr.instruct.UserFunction;
 import net.sf.saxon.expr.parser.ContextItemStaticInfo;
-import net.sf.saxon.expr.parser.ExpressionTool;
 import net.sf.saxon.expr.parser.ExpressionVisitor;
 import net.sf.saxon.expr.parser.RebindingMap;
 import net.sf.saxon.om.*;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.ItemType;
-import net.sf.saxon.value.SequenceExtent;
 
 /**
  * A TailCallLoop wraps the body of a function that contains tail-recursive function calls. On completion
@@ -55,7 +53,7 @@ public final class TailCallLoop extends UnaryExpression {
 
     /*@NotNull*/
     public Expression typeCheck(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
-        typeCheckChildren(visitor, contextInfo);
+        getOperand().typeCheck(visitor, contextInfo);
         return this;
     }
 
@@ -78,7 +76,7 @@ public final class TailCallLoop extends UnaryExpression {
      * Copy an expression. This makes a deep copy.
      *
      * @return the copy of the original expression
-     * @param rebindings
+     * @param rebindings Variables whose binding needs to change
      */
 
     /*@NotNull*/
@@ -94,11 +92,11 @@ public final class TailCallLoop extends UnaryExpression {
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<?> iterate(XPathContext context) throws XPathException {
         final XPathContextMajor cm = (XPathContextMajor) context;
         while (true) {
-            SequenceIterator iter = getBaseExpression().iterate(cm);
-            GroundedValue extent = SequenceExtent.makeSequenceExtent(iter);
+            SequenceIterator<?> iter = getBaseExpression().iterate(cm);
+            GroundedValue<?> extent = iter.materialize();
             TailCallInfo tail = cm.getTailCallInfo();
             if (tail == null) {
                 return extent.iterate();
@@ -184,10 +182,10 @@ public final class TailCallLoop extends UnaryExpression {
      */
 
     /*@Nullable*/
-    private Sequence tailCallDifferentFunction(UserFunction userFunction, XPathContextMajor cm) throws XPathException {
+    private Sequence<?> tailCallDifferentFunction(UserFunction userFunction, XPathContextMajor cm) throws XPathException {
         cm.resetStackFrameMap(userFunction.getStackFrameMap(), userFunction.getArity());
         try {
-            return ExpressionTool.evaluate(userFunction.getBody(), userFunction.getEvaluationMode(), cm, 1);
+            return userFunction.getEvaluator().evaluate(userFunction.getBody(), cm);
         } catch (XPathException err) {
             err.maybeSetLocation(getLocation());
             err.maybeSetContext(cm);
@@ -198,9 +196,6 @@ public final class TailCallLoop extends UnaryExpression {
 
     /**
      * Determine the data type of the items returned by the expression
-     * <p/>
-     * <p/>
-     * /*@NotNull
      */
     public ItemType getItemType() {
         return getBaseExpression().getItemType();
@@ -218,7 +213,7 @@ public final class TailCallLoop extends UnaryExpression {
 
 
 
-    public interface TailCallInfo {};
+    public interface TailCallInfo {}
 
     protected static class TailCallComponent implements TailCallInfo {
         public Component component;
