@@ -12,7 +12,10 @@ import net.sf.saxon.Controller;
 import net.sf.saxon.expr.*;
 import net.sf.saxon.expr.parser.*;
 import net.sf.saxon.lib.NamespaceConstant;
-import net.sf.saxon.om.*;
+import net.sf.saxon.om.GroundedValue;
+import net.sf.saxon.om.SequenceTool;
+import net.sf.saxon.om.StandardNames;
+import net.sf.saxon.om.StructuredQName;
 import net.sf.saxon.query.XQueryFunction;
 import net.sf.saxon.query.XQueryFunctionLibrary;
 import net.sf.saxon.trace.ExpressionPresenter;
@@ -25,7 +28,9 @@ import net.sf.saxon.tree.iter.ManualIterator;
 import net.sf.saxon.type.AnyItemType;
 import net.sf.saxon.type.ItemType;
 import net.sf.saxon.type.TypeHierarchy;
-import net.sf.saxon.value.*;
+import net.sf.saxon.value.Cardinality;
+import net.sf.saxon.value.IntegerValue;
+import net.sf.saxon.value.SequenceType;
 
 import java.util.*;
 
@@ -873,6 +878,10 @@ public class GlobalVariable extends Actor
         presenter.startElement(this instanceof GlobalParam ? "globalParam" : "globalVariable");
         presenter.emitAttribute("name", getVariableQName());
         presenter.emitAttribute("type", getRequiredType().toExportString());
+        ExpressionPresenter.ExportOptions options = (ExpressionPresenter.ExportOptions) presenter.getOptions();
+        if (options.target != null && options.target.equals("JS") && options.targetVersion == 2) {
+            presenter.emitAttribute("typeJ", getRequiredType().toExportString2());
+        }
         presenter.emitAttribute("line", getLineNumber() + "");
         presenter.emitAttribute("module", getSystemId());
         if (getStackFrameMap() != null) {
