@@ -8,8 +8,8 @@
 package net.sf.saxon.lib;
 
 import net.sf.saxon.Configuration;
-import net.sf.saxon.expr.Expression;
-import net.sf.saxon.expr.XPathContext;
+import net.sf.saxon.Controller;
+import net.sf.saxon.expr.*;
 import net.sf.saxon.expr.instruct.*;
 import net.sf.saxon.expr.parser.ExplicitLocation;
 import net.sf.saxon.expr.parser.Location;
@@ -23,13 +23,14 @@ import net.sf.saxon.regex.GeneralUnicodeString;
 import net.sf.saxon.regex.LatinString;
 import net.sf.saxon.regex.UnicodeString;
 import net.sf.saxon.serialize.charcode.UTF16CharacterSet;
-import net.sf.saxon.trace.ContextStackFrame;
-import net.sf.saxon.trace.ContextStackIterator;
 import net.sf.saxon.trace.InstructionInfo;
 import net.sf.saxon.trace.LocationKind;
 import net.sf.saxon.trans.Err;
 import net.sf.saxon.trans.KeyDefinition;
+import net.sf.saxon.trans.Mode;
 import net.sf.saxon.trans.XPathException;
+import net.sf.saxon.trans.rules.BuiltInRuleSet;
+import net.sf.saxon.trans.rules.Rule;
 import net.sf.saxon.tree.AttributeLocation;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.tree.util.Navigator;
@@ -41,7 +42,6 @@ import org.xml.sax.SAXException;
 import javax.xml.transform.SourceLocator;
 import javax.xml.transform.TransformerException;
 import javax.xml.transform.dom.DOMLocator;
-import java.util.Iterator;
 import java.util.List;
 
 /**
@@ -54,9 +54,11 @@ import java.util.List;
 public class StandardErrorListener implements UnfailingErrorListener {
 
     private int recoveryPolicy = Configuration.RECOVER_WITH_WARNINGS;
+    private int requestedRecoveryPolicy = Configuration.RECOVER_WITH_WARNINGS;
     private int warningCount = 0;
     private int maximumNumberOfWarnings = 25;
     private int maxOrdinaryCharacter = 255;
+    private int stackTraceDetail = 2;
     protected transient Logger logger = new StandardLogger();
 
     /**
@@ -78,9 +80,7 @@ public class StandardErrorListener implements UnfailingErrorListener {
         StandardErrorListener sel;
         try {
             sel = this.getClass().newInstance();
-        } catch (InstantiationException e) {
-            sel = new StandardErrorListener();
-        } catch (IllegalAccessException e) {
+        } catch (InstantiationException | IllegalAccessException e) {
             sel = new StandardErrorListener();
         }
         sel.logger = logger;
@@ -122,6 +122,7 @@ public class StandardErrorListener implements UnfailingErrorListener {
 
     public void setRecoveryPolicy(int policy) {
         recoveryPolicy = policy;
+        requestedRecoveryPolicy = policy;
     }
 
     /**
@@ -137,6 +138,17 @@ public class StandardErrorListener implements UnfailingErrorListener {
         return recoveryPolicy;
     }
 
+    /**
+     * Get the requested recovery policy. This returns the original policy, which may differ
+     * from the current policy, in the case where the current policy has switched from "warnings"
+     * to "silent" because too many warnings have been issued
+     * @return the original requested recovery policy
+     */
+
+    public int getRequestedRecoveryPolicy() {
+        return requestedRecoveryPolicy;
+    }
+
     /**
      * Set the maximum number of warnings that are reported; further warnings after this limit
      * are silently ignored
@@ -159,6 +171,33 @@ public class StandardErrorListener implements UnfailingErrorListener {
         return this.maximumNumberOfWarnings;
     }
 
+    /**
+     * Set the level of information to be included in a stack trace when a dynamic
+     * error occurs.
+     * @param level set to 0 (zero) for no stack trace; 1 (one) for a stack trace
+     *              showing the templates and functions being executed; 2 (two) to
+     *              add values of local variables and parameters (available in Saxon-EE only)
+     *              Default is the maximum level available.
+     */
+
+    public void setStackTraceDetail(int level) {
+        stackTraceDetail = level;
+    }
+
+    /**
+     * Get the level of information to be included in a stack trace when a dynamic
+     * error occurs.
+     *
+     * @return 0 (zero) for no stack trace; 1 (one) for a stack trace
+     *              showing the templates and functions being executed; 2 (two) to
+     *              add values of local variables and parameters (available in Saxon-EE only)
+     *              Default is the maximum level available.
+     */
+
+    public int getStackTraceDetail() {
+        return stackTraceDetail;
+    }
+
     /**
      * Set the maximum codepoint value for a character to be considered non-special.
      * Special characters (codepoints above this value) will be expanded in hex for
@@ -340,7 +379,8 @@ public class StandardErrorListener implements UnfailingErrorListener {
 
         if (exception instanceof XPathException) {
             XPathContext context = ((XPathException) exception).getXPathContext();
-            if (context != null && getRecoveryPolicy() != Configuration.RECOVER_SILENTLY) {
+            if (context != null && getRequestedRecoveryPolicy() != Configuration.RECOVER_SILENTLY
+                    && !(context instanceof EarlyEvaluationContext)) {
                 outputStackTrace(logger, context);
             }
         }
@@ -437,10 +477,10 @@ public class StandardErrorListener implements UnfailingErrorListener {
     }
 
     public static String getOffenderListText(ValidationFailure failure) {
-        String message = "";
+        StringBuilder message = new StringBuilder();
         List<NodeInfo> offendingNodes = failure.getOffendingNodes();
         if (!offendingNodes.isEmpty()) {
-            message += "\n  Nodes for which the assertion fails:";
+            message.append("\n  Nodes for which the assertion fails:");
             for (NodeInfo offender : offendingNodes) {
                 String nodeDesc = Type.displayTypeName(offender);
                 if (offender.getNodeKind() == Type.TEXT) {
@@ -457,10 +497,10 @@ public class StandardErrorListener implements UnfailingErrorListener {
                 } else {
                     nodeDesc += " at " + Navigator.getPath(offender);
                 }
-                message += "\n  * " + nodeDesc;
+                message.append("\n  * ").append(nodeDesc);
             }
         }
-        return message;
+        return message.toString();
     }
 
     /**
@@ -471,7 +511,7 @@ public class StandardErrorListener implements UnfailingErrorListener {
      */
 
     protected void outputStackTrace(Logger out, XPathContext context) {
-        printStackTrace(out, context);
+        printStackTrace(context, out, stackTraceDetail);
     }
 
     /**
@@ -581,7 +621,7 @@ public class StandardErrorListener implements UnfailingErrorListener {
                 // no action (can fail with NPE if the expression tree is corrupt)
             }
         }
-        if (systemId != null && systemId.length() != 0) {
+        if (systemId != null && !systemId.isEmpty()) {
             locMessage += (containsLineNumber ? "of " : "in ") + abbreviatePath(systemId) + ':';
         }
         return locMessage;
@@ -795,19 +835,86 @@ public class StandardErrorListener implements UnfailingErrorListener {
     /**
      * Print a stack trace to a specified output destination
      *
-     * @param out     the print stream to which the stack trace will be output
      * @param context the XPath dynamic execution context (which holds the head of a linked
      *                list of context objects, representing the execution stack)
+     * @param out     the print stream to which the stack trace will be output
+     * @param level   the level of detail: 0=none, 1=name and location of function/template,
+     *                2=values of variables
      */
 
-    public static void printStackTrace(Logger out, XPathContext context) {
-        Iterator<ContextStackFrame> iterator = new ContextStackIterator(context);
-        while (iterator.hasNext()) {
-            ContextStackFrame frame = iterator.next();
-            frame.print(out);
+    public static void printStackTrace(XPathContext context, Logger out, int level) {
+        if (level > 0) {
+            int depth = 20;
+            while (depth-- > 0) {
+                if (level > 0) {
+                    Component component = context.getCurrentComponent();
+                    if (component != null) {
+                        if (component.getActor() instanceof Mode) {
+                            Rule rule = context.getCurrentTemplateRule();
+                            if (rule != null) {
+                                StringBuilder sb = new StringBuilder();
+                                Location loc = rule.getPattern().getLocation();
+                                sb.append("  In template rule with match=\"")
+                                        .append(rule.getPattern().toShortString())
+                                        .append("\" ");
+                                if (loc != null && loc.getLineNumber() != -1) {
+                                    sb.append("on line ").append(loc.getLineNumber()).append(" ");
+                                }
+                                if (loc != null && loc.getSystemId() != null) {
+                                    sb.append("of ").append(abbreviatePath(loc.getSystemId()));
+                                }
+                                out.error(sb.toString());
+                            }
+                        } else {
+                            out.error(getLocationMessageText(component.getActor()).replace("$at ","In "));
+                        }
+                    }
+                    context.getStackFrame().getStackFrameMap().showStackFrame(context, out);
+                }
+                while (!(context instanceof XPathContextMajor)) {
+                    context = context.getCaller();
+                }
+
+                ContextOriginator originator = ((XPathContextMajor) context).getOrigin();
+                if (originator == null || originator instanceof Controller) {
+                    return;
+                } else {
+                    out.error("     invoked by " + showOriginator(originator));
+                }
+                context = context.getCaller();
+            }
         }
     }
 
+    private static String showOriginator(ContextOriginator originator) {
+        StringBuilder sb = new StringBuilder();
+        if (originator instanceof ApplyTemplates) {
+            sb.append("xsl:apply-templates");
+        } else if (originator instanceof NextMatch) {
+            sb.append("xsl:next-match");
+        } else if (originator instanceof ApplyImports) {
+            sb.append("xsl:apply-imports");
+        } else if (originator instanceof UserFunctionCall) {
+            sb.append("function call");
+        } else if (originator instanceof CallTemplate) {
+            sb.append("xsl:call-template");
+        } else if (originator instanceof Controller) {
+            sb.append("external application");
+        } else if (originator instanceof BuiltInRuleSet) {
+            sb.append("built-in template rule (" + ((BuiltInRuleSet)originator).getName() + ")");
+        } else {
+            sb.append("unknown caller (" + originator.getClass() + ")");
+        }
+        if (originator instanceof Locatable) {
+            Location loc = ((Locatable)originator).getLocation();
+            if (loc.getLineNumber() != -1) {
+                sb.append(" at ").append(loc.getSystemId() == null ? "line " : (loc.getSystemId() + "#"));
+                sb.append(loc.getLineNumber());
+            }
+        }
+        return sb.toString();
+    }
+
     /**
      * Expand any special characters appearing in a message. Special characters
      * will be output as themselves, followed by a hex codepoint in the form [xHHHHH].
