@@ -21,7 +21,6 @@ import net.sf.saxon.serialize.SerializationProperties;
 import net.sf.saxon.trans.SaxonErrorCode;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.trans.XsltController;
-import net.sf.saxon.tree.tiny.Statistics;
 import net.sf.saxon.tree.tiny.TinyBuilder;
 
 import javax.xml.transform.ErrorListener;
@@ -251,22 +250,7 @@ abstract class AbstractXsltTransformer {
     }
 
     /**
-     * Ask whether assertions (xsl:assert instructions) have been enabled at run time. By default
-     * they are disabled at compile time. If assertions are enabled at compile time, then by
-     * default they will also be enabled at run time; but they can be disabled at run time by
-     * specific request. At compile time, assertions can be enabled for some packages and
-     * disabled for others; at run-time, they can only be enabled or disabled globally.
-     *
-     * @return true if assertions are enabled at run time
-     * @since 9.7
-     */
-
-    public boolean isAssertionsEnabled() {
-        return controller.isAssertionsEnabled();
-    }
-
-    /**
-     * Ask whether assertions (xsl:assert instructions) have been enabled at run time. By default
+     * Say whether assertions (xsl:assert instructions) should be enabled at run time. By default
      * they are disabled at compile time. If assertions are enabled at compile time, then by
      * default they will also be enabled at run time; but they can be disabled at run time by
      * specific request. At compile time, assertions can be enabled for some packages and
@@ -281,6 +265,21 @@ abstract class AbstractXsltTransformer {
         controller.setAssertionsEnabled(enabled);
     }
 
+    /**
+     * Ask whether assertions (xsl:assert instructions) have been enabled at run time. By default
+     * they are disabled at compile time. If assertions are enabled at compile time, then by
+     * default they will also be enabled at run time; but they can be disabled at run time by
+     * specific request. At compile time, assertions can be enabled for some packages and
+     * disabled for others; at run-time, they can only be enabled or disabled globally.
+     *
+     * @return true if assertions are enabled at run time
+     * @since 9.7
+     */
+
+    public boolean isAssertionsEnabled() {
+        return controller.isAssertionsEnabled();
+    }
+
     /**
      * Set a TraceListener to be notified of all events occurring during the transformation.
      * This will only be effective if the stylesheet was compiled with trace code enabled
@@ -493,6 +492,7 @@ abstract class AbstractXsltTransformer {
      */
 
     protected Receiver getReceivingTransformer(XsltController controller, GlobalParameterSet parameters, Destination finalDestination) throws SaxonApiException {
+        Configuration config = controller.getConfiguration();
         try {
             controller.initializeController(parameters);
         } catch (XPathException e) {
@@ -509,7 +509,7 @@ abstract class AbstractXsltTransformer {
         } else {
             final Builder sourceTreeBuilder = controller.makeBuilder();
             if (sourceTreeBuilder instanceof TinyBuilder) {
-                ((TinyBuilder) sourceTreeBuilder).setStatistics(Statistics.SOURCE_DOCUMENT_STATISTICS);
+                ((TinyBuilder) sourceTreeBuilder).setStatistics(config.getTreeStatistics().SOURCE_DOCUMENT_STATISTICS);
             }
             Receiver stripper = controller.makeStripper(sourceTreeBuilder);
             if (controller.isStylesheetStrippingTypeAnnotations()) {
