@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,8 +8,10 @@
 package net.sf.saxon.s9api;
 
 import net.sf.saxon.Configuration;
+import net.sf.saxon.expr.parser.OptimizerOptions;
 import net.sf.saxon.lib.ErrorGatherer;
 import net.sf.saxon.lib.ModuleURIResolver;
+import net.sf.saxon.lib.StringCollator;
 import net.sf.saxon.pattern.NodeKindTest;
 import net.sf.saxon.query.StaticQueryContext;
 import net.sf.saxon.trans.XPathException;
@@ -23,14 +25,12 @@ import java.util.List;
 /**
  * An XQueryCompiler object allows XQuery 1.0 queries to be compiled. The compiler holds information that
  * represents the static context for the compilation.
- * <p/>
  * <p>To construct an XQueryCompiler, use the factory method {@link Processor#newXQueryCompiler}.</p>
- * <p/>
  * <p>An XQueryCompiler may be used repeatedly to compile multiple queries. Any changes made to the
  * XQueryCompiler (that is, to the static context) do not affect queries that have already been compiled.
  * An XQueryCompiler may in principle be used concurrently in multiple threads, but in practice this
  * is best avoided because all instances will share the same ErrorListener and it will therefore be
- * difficult to establish which error messages are associated with each compilation..</p>
+ * difficult to establish which error messages are associated with each compilation.</p>
  *
  * @since 9.0
  */
@@ -315,6 +315,7 @@ public class XQueryCompiler {
         if (!"3.1".equals(value)) {
             env.getErrorListener().warning(new XPathException("XQuery version ignored: only \"3.1\" is recognized"));
         }
+        env.setLanguageVersion(31);
     }
 
     /**
@@ -348,27 +349,11 @@ public class XQueryCompiler {
         env.declareNamespace(prefix, uri);
     }
 
-    /**
-     * Bind a collation URI to a collation
-     *
-     * @param uri       the absolute collation URI
-     * @param collation a {@link java.text.Collator} object that implements the required collation
-     * @throws IllegalArgumentException if an attempt is made to rebind the standard URI
-     *                                  for the Unicode codepoint collation
-     * @since 9.4
-     * @deprecated since 9.6. Collations are now held globally. If this method is called, the effect
-     * is to update the pool of collations held globally by the Processor.
-     */
-
-    public void declareCollation(String uri, final java.text.Collator collation) {
-        getProcessor().declareCollation(uri, collation);
-    }
-
     /**
      * Declare the default collation
      *
      * @param uri the absolute URI of the default collation. This URI must have been bound to a collation
-     *            using the method {@link #declareCollation(String, java.text.Collator)}, or it must be
+     *            using the method {@link Configuration#registerCollation(String, StringCollator)}, or it must be
      *            one that is recognized implicitly, such as a UCA collation
      * @throws NullPointerException  if the collation URI is null
      * @throws IllegalStateException if the collation URI has not been registered, unless it is the standard
@@ -405,6 +390,38 @@ public class XQueryCompiler {
         return requiredContextItemType;
     }
 
+    /**
+     * Request fast compilation. Fast compilation will generally be achieved at the expense of run-time performance
+     * and quality of diagnostics. Fast compilation is a good trade-off if (a) the expression is known to be correct,
+     * and (b) once compiled, the expression is only executed once against a document of modest size.
+     * <p><i>The current implementation is equivalent to switching off all optimizations. Setting this option, however,
+     * indicates an intent rather than a mechanism, and the implementation details may change in future to reflect
+     * the intent.</i></p>
+     *
+     * @param fast set to true to request fast compilation; set to false to revert to the optimization options
+     *             defined in the Configuration.
+     * @since 9.9
+     */
+
+    public void setFastCompilation(boolean fast) {
+        if (fast) {
+            env.setOptimizerOptions(new OptimizerOptions(0));
+        } else {
+            env.setOptimizerOptions(getProcessor().getUnderlyingConfiguration().getOptimizerOptions());
+        }
+    }
+
+    /**
+     * Ask if fast compilation has been enabled.
+     *
+     * @return true if fast compilation has been enabled
+     * @since 9.9
+     */
+
+    public boolean isFastCompilation() {
+        return env.getOptimizerOptions().getOptions() == 0;
+    }
+
     /**
      * Compile a library module supplied as a string. The code generated by compiling the library is available
      * for importing by all subsequent compilations using the same XQueryCompiler; it is identified by an
@@ -445,19 +462,13 @@ public class XQueryCompiler {
      */
 
     public void compileLibrary(File query) throws SaxonApiException, IOException {
-        FileInputStream stream = null;
-        try {
-            stream = new FileInputStream(query);
+        try (FileInputStream stream = new FileInputStream(query)) {
             String savedBaseUri = env.getBaseURI();
             env.setBaseURI(query.toURI().toString());
             env.compileLibrary(stream, encoding);
             env.setBaseURI(savedBaseUri);
         } catch (XPathException e) {
             throw new SaxonApiException(e);
-        } finally {
-            if (stream != null) {
-                stream.close();
-            }
         }
     }
 
@@ -477,9 +488,7 @@ public class XQueryCompiler {
     public void compileLibrary(Reader query) throws SaxonApiException {
         try {
             env.compileLibrary(query);
-        } catch (XPathException e) {
-            throw new SaxonApiException(e);
-        } catch (IOException e) {
+        } catch (XPathException | IOException e) {
             throw new SaxonApiException(e);
         }
     }
@@ -503,18 +512,14 @@ public class XQueryCompiler {
     public void compileLibrary(InputStream query) throws SaxonApiException {
         try {
             env.compileLibrary(query, encoding);
-        } catch (XPathException e) {
-            throw new SaxonApiException(e);
-        } catch (IOException e) {
+        } catch (XPathException | IOException e) {
             throw new SaxonApiException(e);
         }
     }
 
     /**
      * Compile a query supplied as a string.
-     * <p/>
      * <p>The base URI of the query should be supplied by calling {@link #setBaseURI(java.net.URI)} </p>
-     * <p/>
      * <p>If the query contains static errors, these will be notified to the registered {@link ErrorListener}.
      * More than one static error may be reported. If any static errors have been reported, this method
      * will exit with an exception, but the exception will only contain a summary message. The default
@@ -547,9 +552,7 @@ public class XQueryCompiler {
      */
 
     public XQueryExecutable compile(File query) throws SaxonApiException, IOException {
-        FileInputStream stream = null;
-        try {
-            stream = new FileInputStream(query);
+        try (FileInputStream stream = new FileInputStream(query)) {
             String savedBaseUri = env.getBaseURI();
             env.setBaseURI(query.toURI().toString());
             XQueryExecutable exec =
@@ -558,10 +561,6 @@ public class XQueryCompiler {
             return exec;
         } catch (XPathException e) {
             throw new SaxonApiException(e);
-        } finally {
-            if (stream != null) {
-                stream.close();
-            }
         }
     }
 
@@ -572,11 +571,10 @@ public class XQueryCompiler {
      * @param query the input stream on which the query is supplied. This will be consumed by this method
      * @return an XQueryExecutable representing the compiled query
      * @throws SaxonApiException if the query compilation fails with a static error
-     * @throws IOException       if the file does not exist or cannot be read
      * @since 9.1
      */
 
-    public XQueryExecutable compile(InputStream query) throws SaxonApiException, IOException {
+    public XQueryExecutable compile(InputStream query) throws SaxonApiException {
         try {
             return new XQueryExecutable(processor, env.compileQuery(query, encoding));
         } catch (XPathException e) {
@@ -621,10 +619,12 @@ public class XQueryCompiler {
      * List of errors. The caller should supply an empty list before calling Compile; the processor will then populate
      * the list with error information obtained during the compilation. Each error will be included as an object of type StaticError.
      * If no error list is supplied by the caller, error information will be written to the standard error stream.
-     * <p>
-     * By supplying a custom List with a user-written add() method, it is possible to intercept error conditions as they occur.
+     * <p>By supplying a custom List with a user-written add() method, it is possible to intercept error conditions as they occur.</p>
+     * <p>Calling this method overwrites any previous call to {@link #setErrorListener}, as well as any
+     * previous call to {@code #setErrorList}.</p>
      *
-     * @param errorList
+     * @param errorList a (typically empty) list that will be populated with {@link StaticError} objects giving details
+     *                  of any static errors found in the query.
      */
     public void setErrorList(List<StaticError> errorList) {
         env.setErrorListener(new ErrorGatherer(errorList));
