@@ -25,10 +25,7 @@ import net.sf.saxon.type.BuiltInAtomicType;
 import net.sf.saxon.type.SchemaType;
 import net.sf.saxon.type.Untyped;
 
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.Iterator;
-import java.util.List;
+import java.util.*;
 
 
 /**
@@ -44,7 +41,7 @@ public class LiteralResultElement extends StyleElement {
     private Expression[] attributeValues;
     private int numberOfAttributes;
     private boolean toplevel;
-    private List<NamespaceBinding> namespaceCodes = new ArrayList<NamespaceBinding>();
+    private List<NamespaceBinding> namespaceCodes = new ArrayList<>();
     private StructuredQName[] attributeSets;
     /*@Nullable*/ private SchemaType schemaType = null;
     private int validation = Validation.STRIP;
@@ -92,7 +89,8 @@ public class LiteralResultElement extends StyleElement {
             for (int i = 0; i < num; i++) {
 
                 int fp = atts.getFingerprint(i);
-                String attURI = namePool.getURI(fp);
+                NodeName name = atts.getNodeName(i);
+                String attURI = name.getURI();
 
                 if (attURI.equals(NamespaceConstant.XSLT)) {
 
@@ -129,7 +127,7 @@ public class LiteralResultElement extends StyleElement {
                                              Err.wrap(atts.getNodeName(i).getDisplayName(), Err.ATTRIBUTE), "XTSE0805");
                     }
                 } else {
-                    attributeNames[numberOfAttributes] = new FingerprintedQName(atts.getPrefix(i), atts.getURI(i), atts.getLocalName(i), fp);
+                    attributeNames[numberOfAttributes] = name;
                     Expression exp = makeAttributeValueTemplate(atts.getValue(i), i);
                     attributeValues[numberOfAttributes] = exp;
                     numberOfAttributes++;
@@ -247,7 +245,7 @@ public class LiteralResultElement extends StyleElement {
                     NodeName alias = anameCode;
                     String attURI = anameCode.getURI();
 
-                    if (attURI.length() != 0) {    // attribute has a namespace prefix
+                    if (!attURI.isEmpty()) {    // attribute has a namespace prefix
                         NamespaceBinding newBinding = sheet.getNamespaceAlias(attURI);
                         if (newBinding != null && !newBinding.getURI().equals(attURI)) {
                             alias = new FingerprintedQName(
@@ -299,7 +297,7 @@ public class LiteralResultElement extends StyleElement {
             return null;
         }
 
-        NamespaceBinding[] bindings = namespaceCodes.toArray(new NamespaceBinding[namespaceCodes.size()]);
+        NamespaceBinding[] bindings = namespaceCodes.toArray(NamespaceBinding.EMPTY_ARRAY);
         FixedElement inst = new FixedElement(
                 resultNodeName,
                 bindings,
