@@ -68,7 +68,6 @@ public class StylesheetPackage extends PackageData {
     // table of imported schemas. The members of this set are strings holding the target namespace.
     protected Set<String> schemaIndex = new HashSet<>(10);
 
-
     private FunctionLibraryList functionLibrary;
     private XQueryFunctionLibrary queryFunctions;
     private ExecutableFunctionLibrary overriding;
@@ -496,7 +495,7 @@ public class StylesheetPackage extends PackageData {
      */
 
     public void allocateBinderySlots() {
-        SlotManager slotManager = new SlotManager();
+        SlotManager slotManager = getConfiguration().makeSlotManager();
         for (Component c : componentIndex.values()) {
             registerGlobalVariable(c, slotManager);
         }
@@ -673,7 +672,9 @@ public class StylesheetPackage extends PackageData {
             newC.setBaseComponent(oldC);
 
             if (overrides.contains(name)) {
-                // TODO: overrides is all the overrides, not only those for this xsl:use-package
+                // Note: overrides is all the overrides, not only those for this xsl:use-package;
+                // but we have already checked that xsl:override declarations match something in the
+                // right package.
                 overriddenComponents.put(name, newC);
                 if (newV != Visibility.ABSTRACT) {
                     abstractComponents.remove(name);
@@ -1082,8 +1083,8 @@ public class StylesheetPackage extends PackageData {
                 }
             }
             throw new XPathException(
-                "The package is not executable, because it contains abstract components: " +
-                    buff.toString(), "XTSE3080");
+                    "The package is not executable, because it contains abstract components: " +
+                            buff, "XTSE3080");
         }
     }
 
