@@ -365,21 +365,6 @@ public class XSLFunction extends StyleElement implements StylesheetComponent {
         top.indexFunction(decl);
     }
 
-    /**
-     * Notify all references to this function of the data type.
-     *
-     * @throws XPathException
-     */
-
-    public void fixupReferences() throws XPathException {
-//        for (UserFunctionReference reference : references) {
-//            if (reference instanceof UserFunctionCall) {
-//                ((UserFunctionCall) reference).setStaticType(resultType);
-//            }
-//        }
-        super.fixupReferences();
-    }
-
     public void validate(ComponentDeclaration decl) throws XPathException {
 
         stackFrameMap = getConfiguration().makeSlotManager();
@@ -401,7 +386,7 @@ public class XSLFunction extends StyleElement implements StylesheetComponent {
      * all references to the function to the executable representation
      * (a UserFunction object)
      *
-     * @throws XPathException
+     * @throws XPathException if compilation fails
      */
 
     public void compileDeclaration(Compilation compilation, ComponentDeclaration decl) throws XPathException {
@@ -451,8 +436,9 @@ public class XSLFunction extends StyleElement implements StylesheetComponent {
         compiledFunction.setBody(exp2);
 
         // Assess the streamability of the function body
+        Optimizer optimizer = visitor.getConfiguration().obtainOptimizer();
         if (streamability.isStreaming()) {
-            visitor.getConfiguration().obtainOptimizer().assessFunctionStreamability(this, compiledFunction);
+            optimizer.assessFunctionStreamability(this, compiledFunction);
         }
 
         allocateLocalSlots(exp2);
@@ -460,13 +446,13 @@ public class XSLFunction extends StyleElement implements StylesheetComponent {
             compiledFunction.setBody(exp2);
         }
 
-        if (!streamability.isStreaming()) {
+        OptimizerOptions options = getCompilation().getCompilerInfo().getOptimizerOptions();
+        if (options.isSet(OptimizerOptions.TAIL_CALLS) && !streamability.isStreaming()) {
             int tailCalls = ExpressionTool.markTailFunctionCalls(exp2, getObjectName(), getNumberOfArguments());
             if (tailCalls != 0) {
                 compiledFunction.setTailRecursive(tailCalls > 0, tailCalls > 1);
                 exp2 = compiledFunction.getBody();
                 compiledFunction.setBody(new TailCallLoop(compiledFunction, exp2));
-                //compiledFunction.getBody().setPostureAndSweep(exp2.getPostureAndSweepIfKnown());
             }
         }
 
