@@ -630,10 +630,9 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
             //System.err.println("Deferred - " + ++lazy);
             return;
         }
-        if (compiledNamedTemplate != null) {
-            compiledNamedTemplate.resetLocalParams();
+        if (compilation.getCompilerInfo().getOptimizerOptions().isSet(OptimizerOptions.TAIL_CALLS)) {
+            isTailRecursive = markTailCalls();
         }
-        isTailRecursive = markTailCalls();
         Expression body = compileSequenceConstructor(compilation, decl, true);
         body.restoreParentPointers();
         RetainedStaticContext rsc = makeRetainedStaticContext();
@@ -1018,8 +1017,7 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
                 body.explain(err);
             }
             body.restoreParentPointers();
-            // TODO: generate byte code?
-            if (config.isDeferredByteCode(Configuration.XSLT)) {
+            if (config.isDeferredByteCode(Configuration.XSLT) && !isTailRecursive) {
                 Optimizer opt = config.obtainOptimizer();
                 int evaluationModes = Expression.ITERATE_METHOD | Expression.PROCESS_METHOD;
                 compiledNamedTemplate.setBody(opt.makeByteCodeCandidate(compiledNamedTemplate, body, diagnosticId, evaluationModes));
@@ -1068,7 +1066,7 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
                             }
                         }
 
-                        if (visitor.getConfiguration().isDeferredByteCode(Configuration.XSLT)) {
+                        if (visitor.getConfiguration().isDeferredByteCode(Configuration.XSLT) && !isTailRecursive) {
                             int evaluationModes = Expression.ITERATE_METHOD | Expression.PROCESS_METHOD;
                             compiledTemplateRule.setBody(opt.makeByteCodeCandidate(compiledTemplateRule, templateRuleBody, diagnosticId, evaluationModes));
                         }
