@@ -137,6 +137,9 @@ public class Err {
     }
 
     public static CharSequence depictSequence(Sequence<?> seq) {
+        if (seq == null) {
+            return "(*null*)";
+        }
         try {
             GroundedValue<?> val = seq.materialize();
             if (val.getLength() == 0) {
@@ -146,7 +149,7 @@ public class Err {
             } else {
                 return depictSequenceStart(val.iterate(), 3, val.getLength());
             }
-        } catch (XPathException e) {
+        } catch (Exception e) {
             return "(*unreadable*)";
         }
     }
@@ -191,6 +194,9 @@ public class Err {
      */
 
     public static String abbreviateURI(String uri) {
+        if (uri==null) {
+            return "";
+        }
         int lastSlash = (uri.endsWith("/") ? uri.substring(0, uri.length()-1) : uri).lastIndexOf('/');
         if (lastSlash < 0) {
             if (uri.length() > 15) {
