@@ -18,6 +18,7 @@ import net.sf.saxon.tree.util.NamespaceIterator;
 import net.sf.saxon.tree.util.Navigator;
 import net.sf.saxon.type.*;
 import net.sf.saxon.value.AtomicValue;
+import net.sf.saxon.z.IntHashMap;
 
 
 /**
@@ -59,8 +60,18 @@ public class TinyElementImpl extends TinyParentNodeImpl {
      */
 
     public String getBaseURI() {
-        return Navigator.getBaseURI(this,
-                                    n -> ((TinyElementImpl) n).getTree().isTopWithinEntity(((TinyElementImpl) n).getNodeNumber()));
+        synchronized (tree) {
+            if (tree.knownBaseUris == null) {
+                tree.knownBaseUris = new IntHashMap<>();
+            }
+            String uri = tree.knownBaseUris.get(nodeNr);
+            if (uri == null) {
+                uri = Navigator.getBaseURI(this,
+                                           n -> tree.isTopWithinEntity(((TinyElementImpl) n).getNodeNumber()));
+                tree.knownBaseUris.put(nodeNr, uri);
+            }
+            return uri;
+        }
     }
 
     /**
@@ -153,6 +164,37 @@ public class TinyElementImpl extends TinyParentNodeImpl {
         }
     }
 
+    /**
+     * Ask whether the element is the root of a subtree in which no descendant element
+     * has any local namespace declarations or undeclarations; that is, all elements
+     * in the subtree have the same in-scope namespace bindings.
+     * @return true if it is known that no descendant elements have in-scope namespaces
+     * different from those of this element.
+     */
+
+    public boolean hasUniformNamespaces() {
+        int nr = nodeNr;
+        int ns = tree.beta[nr];
+        TinyElementImpl anc = this;
+        while (ns == -1) {
+            TinyNodeImpl parent = anc.getParent();
+            if (parent instanceof TinyDocumentImpl) {
+                return !tree.usesNamespaces;
+            } else {
+                nr = parent.nodeNr;
+                ns = tree.beta[nr];
+                anc = (TinyElementImpl)parent;
+            }
+        }
+        // Find the first namespace binding with a different parent
+        while (ns < tree.numberOfNamespaces && tree.namespaceParent[ns] == nr) {
+            ns++;
+        }
+        // Return true if none is found, or if the element owning this namespace binding
+        // is outside the subtree
+        return ns >= tree.numberOfNamespaces || !isAncestorOrSelf(tree.getNode(tree.namespaceParent[ns]));
+    }
+
 
     /**
      * Get the string value of a given attribute of this node
@@ -173,9 +215,10 @@ public class TinyElementImpl extends TinyParentNodeImpl {
         }
         NamePool pool = getNamePool();
         while (a < tree.numberOfAttributes && tree.attParent[a] == nodeNr) {
-            int fp = tree.attCode[a] & NamePool.FP_MASK;
+            int nc = tree.attCode[a];
             // Avoid allocating a name code for an ad-hoc request
-            if (pool.getLocalName(fp).equals(local) && pool.getURI(fp).equals(uri)) {
+            StructuredQName name = pool.getUnprefixedQName(nc);
+            if (name.getLocalPart().equals(local) && name.hasURI(uri)) {
                 return tree.attValue[a].toString();
             }
             a++;
@@ -205,6 +248,17 @@ public class TinyElementImpl extends TinyParentNodeImpl {
         return null;
     }
 
+    private int subtreeSize() {
+        int next = tree.next[nodeNr];
+        while (next < nodeNr) {
+            if (next < 0) {
+                return tree.numberOfNodes - nodeNr;
+            }
+            next = tree.next[next];
+        }
+        return nodeNr - next;
+    }
+
     /**
      * Copy this node to a given receiver
      *
@@ -214,55 +268,22 @@ public class TinyElementImpl extends TinyParentNodeImpl {
 
     public void copy(/*@NotNull*/ Receiver receiver, int copyOptions, Location location) throws XPathException {
 
-        boolean typed = CopyOptions.includes(copyOptions, CopyOptions.TYPE_ANNOTATIONS);
+        boolean copyTypes = CopyOptions.includes(copyOptions, CopyOptions.TYPE_ANNOTATIONS);
 
-        // Fast path for copying to another TinyTree
-        if (TinyTree.useFastCopy) {
-            boolean copyTypes = typed;
-            Receiver r1 = receiver;
-            if (isSkipValidator(r1)) {
-                copyTypes = false;
-                r1 = ((ProxyReceiver) r1).getNextReceiver();
-            }
-            if (r1 instanceof ComplexContentOutputter && !copyTypes) {
-                Receiver r2 = ((ComplexContentOutputter) r1).getReceiver();
-                if (r2 instanceof NamespaceReducer) {
-                    Receiver r3 = ((NamespaceReducer) r2).getNextReceiver();
-                    if (r3 instanceof TinyBuilder) {
-                        ((ComplexContentOutputter) r1).beforeBulkCopy();
-                        TinyBuilder target = (TinyBuilder) r3;
-                        target.bulkCopy(getTree(), nodeNr);
-                        ((ComplexContentOutputter) r1).afterBulkCopy();
-                        return;
-                    }
-                }
-            }
-        } else if (TinyTree.useGraft && (copyOptions & CopyOptions.FOR_UPDATE) == 0) {
-            boolean copyTypes = typed;
-            Receiver r1 = receiver;
-            if (isSkipValidator(r1)) {
-                copyTypes = false;
-                r1 = ((ProxyReceiver) r1).getNextReceiver();
-            }
-            if (r1 instanceof ComplexContentOutputter && !copyTypes) {
-                Receiver r2 = ((ComplexContentOutputter) r1).getReceiver();
-                if (r2 instanceof NamespaceReducer) {
-                    if (!((NamespaceReducer)r2).isDisinheritingNamespaces()) {
-                        Receiver r3 = ((NamespaceReducer) r2).getNextReceiver();
-                        if (r3 instanceof TinyBuilder && ((TinyBuilder) r3).isPositionedAtElement()) {
-                            ((ComplexContentOutputter) r1).beforeBulkCopy();
-                            TinyBuilder target = (TinyBuilder) r3;
-                            boolean copyNamespaces = CopyOptions.includes(copyOptions, CopyOptions.ALL_NAMESPACES);
-                            target.graft(this, copyNamespaces);
-                            ((ComplexContentOutputter) r1).afterBulkCopy();
-                            return;
-                        }
-                    }
-                }
-            }
+        boolean fastCopied = tryBulkCopy(copyOptions, receiver);
+        if (fastCopied) {
+            return;
         }
 
-        // control vars
+//        boolean grafted = tryGraft(copyOptions, receiver);
+//        if (grafted) {
+//            Instrumentation.count("GRAFT");
+//            Instrumentation.count("GRAFT NODES ", Count.count(iterateAxis(AxisInfo.DESCENDANT_OR_SELF, NodeKindTest.ELEMENT)));
+//            return;
+//        }
+//        Instrumentation.count("NON-GRAFT");
+
+
         short level = -1;
         boolean closePending = false;
         short startLevel = tree.depth[nodeNr];
@@ -278,6 +299,7 @@ public class TinyElementImpl extends TinyParentNodeImpl {
         SchemaType elementType = Untyped.getInstance();
         SimpleType attributeType = BuiltInAtomicType.UNTYPED_ATOMIC;
 
+
         do {
 
             // determine node depth
@@ -303,7 +325,7 @@ public class TinyElementImpl extends TinyParentNodeImpl {
                 case Type.TEXTUAL_ELEMENT: {
 
                     // start element
-                    if (typed) {
+                    if (copyTypes) {
                         elementType = tree.getSchemaType(next);
                         if (disallowNamespaceSensitiveContent) {
                             try {
@@ -346,7 +368,7 @@ public class TinyElementImpl extends TinyParentNodeImpl {
                     } else {
                         // there is an element to close
                         closePending = true;
-                        
+
                         // output namespaces
                         if ((copyOptions & CopyOptions.SOME_NAMESPACES) != 0 && tree.usesNamespaces) {
                             String defaultNS = null;
@@ -389,7 +411,7 @@ public class TinyElementImpl extends TinyParentNodeImpl {
                             while (att < tree.numberOfAttributes && tree.attParent[att] == next) {
                                 int attCode = tree.attCode[att];
                                 int attfp = attCode & NamePool.FP_MASK;
-                                if (typed) {
+                                if (copyTypes) {
                                     attributeType = tree.getAttributeType(att);
                                     if (disallowNamespaceSensitiveContent) {
                                         try {
@@ -402,7 +424,7 @@ public class TinyElementImpl extends TinyParentNodeImpl {
                                     }
                                 }
                                 String attPrefix = tree.prefixPool.getPrefix(attCode >> 20);
-                                receiver.attribute(new CodedName(attfp, attPrefix, pool), attributeType, tree.attValue[att], location, 0);
+                                receiver.attribute(new CodedName(attfp, attPrefix, pool), attributeType, tree.attValue[att], location, ReceiverOptions.NOT_A_DUPLICATE);
                                 att++;
                             }
                         }
@@ -484,6 +506,71 @@ public class TinyElementImpl extends TinyParentNodeImpl {
         }
     }
 
+    private boolean tryGraft(int copyOptions, Receiver out) throws XPathException {
+        if (subtreeSize() < 100) {
+            return false;
+        }
+        if (TinyTree.useGraft &&
+                (copyOptions & CopyOptions.FOR_UPDATE) == 0 &&
+                (copyOptions & CopyOptions.ALL_NAMESPACES) != 0 &&
+                // don't allow a subtree that already contains external nodes to be grafted to another tree
+                tree.externalNodes == null) {
+            if (isSkipValidator(out)) {
+                return false;
+                // Can't currently use grafting copy with validation="strip" because of the
+                // complications of ID and IDREF attributes (test case copy-5034)
+                //r1 = ((ProxyReceiver) r1).getNextReceiver();
+            }
+            if (tree.isTyped()) {
+                return false;
+            }
+            if (out instanceof SequenceWriter && ((SequenceWriter) out).isReadyForGrafting()) {
+                ((SequenceWriter) out).graftElementNode(this, copyOptions);
+                return true;
+            } else if (out instanceof ComplexContentOutputter) {
+                if (((ComplexContentOutputter) out).isReadyForGrafting()) {
+                    ((ComplexContentOutputter) out).graftElementNode(this, copyOptions);
+                    return true;
+                }
+            }
+        }
+        return false;
+    }
+
+    private boolean tryBulkCopy(int copyOptions, Receiver out) throws XPathException {
+        // Fast path for copying to another TinyTree
+        if (TinyTree.useBulkCopy &&
+                    (copyOptions & CopyOptions.FOR_UPDATE) == 0 &&
+                    (copyOptions & CopyOptions.ALL_NAMESPACES) != 0 &&
+                    // don't allow a subtree that already contains external nodes to be grafted to another tree
+                    tree.externalNodes == null) {
+            if (isSkipValidator(out)) {
+                return false;
+                // Can't currently use grafting copy with validation="strip" because of the
+                // complications of ID and IDREF attributes (test case copy-5034)
+                //r1 = ((ProxyReceiver) r1).getNextReceiver();
+            }
+            if (tree.isTyped()) {
+                return false;
+            }
+            if (nodeNr == tree.numberOfNodes - 1) {
+                // Bug 4089
+                // Not sure why this case fails, but it crashes out, and this is the simplest workaround
+                return false;
+            }
+            if (out instanceof SequenceWriter && ((SequenceWriter) out).isReadyForBulkCopy()) {
+                ((SequenceWriter) out).bulkCopyElementNode(this, copyOptions);
+                return true;
+            } else if (out instanceof ComplexContentOutputter) {
+                if (((ComplexContentOutputter) out).isReadyForBulkCopy()) {
+                    ((ComplexContentOutputter) out).bulkCopyElementNode(this, copyOptions);
+                    return true;
+                }
+            }
+        }
+        return false;
+    }
+
     /**
      * Check whether the content of an element is namespace-sensitive
      *
