@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,7 +7,12 @@
 
 package net.sf.saxon.value;
 
+import com.saxonica.ee.extfn.ExperimentalSequenceTypeMarshaller;
+import com.saxonica.ee.extfn.ExperimentalXPathCompiler;
 import net.sf.saxon.expr.StaticProperty;
+import net.sf.saxon.expr.parser.RoleDiagnostic;
+import net.sf.saxon.ma.map.MapItem;
+import net.sf.saxon.om.GroundedValue;
 import net.sf.saxon.om.Item;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.om.SequenceIterator;
@@ -16,6 +21,8 @@ import net.sf.saxon.pattern.NodeKindTest;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.*;
 
+import java.util.Optional;
+
 /**
  * SequenceType: a sequence type consists of a primary type, which indicates the type of item,
  * and a cardinality, which indicates the number of occurrences permitted. Where the primary type
@@ -365,6 +372,12 @@ public final class SequenceType {
     public static final SequenceType OPTIONAL_FUNCTION_ITEM =
             makeSequenceType(AnyFunctionType.getInstance(), StaticProperty.ALLOWS_ZERO_OR_ONE);
 
+    /**
+     * A type that allows a sequence of zero or mode function items
+     */
+    public static final SequenceType FUNCTION_ITEM_SEQUENCE =
+            makeSequenceType(AnyFunctionType.getInstance(), StaticProperty.ALLOWS_ZERO_OR_MORE);
+
 
     /**
      * A type that only permits the empty sequence
@@ -465,12 +478,9 @@ public final class SequenceType {
      *                        won't happen if the sequence is grounded)
      */
 
-    public boolean matches(Sequence value, TypeHierarchy th) throws XPathException {
-        if (value instanceof Item) {
-            return primaryType.matches((Item) value, th);
-        }
+    public boolean matches(Sequence<?> value, TypeHierarchy th) throws XPathException {
         int count = 0;
-        SequenceIterator iter = value.iterate();
+        SequenceIterator<?> iter = value.iterate();
         Item item;
         while ((item = iter.next()) != null) {
             count++;
@@ -482,6 +492,46 @@ public final class SequenceType {
                          || count > 1 && !Cardinality.allowsMany(cardinality));
     }
 
+    /**
+     * Get extra diagnostic information about why a supplied item does not conform to this
+     * item type, if available. If extra information is returned, it should be in the form of a complete
+     * sentence, minus the closing full stop. No information should be returned for obvious cases.
+     *
+     * @param value the value which has been found not to match this sequence type
+     * @param th the TypeHierarchy cache
+     */
+
+    public Optional<String> explainMismatch(GroundedValue value, TypeHierarchy th) {
+        try {
+            int count = 0;
+            SequenceIterator iter = value.iterate();
+            Item item;
+            while ((item = iter.next()) != null) {
+                count++;
+                if (!primaryType.matches(item, th)) {
+                    String s = "The " + RoleDiagnostic.ordinal(count) + " item is not an instance of the required type";
+                    Optional<String> more = primaryType.explainMismatch(item, th);
+                    if (more.isPresent()) {
+                        s = count == 1 ? more.get() : s + ". " + more.get();
+                    } else {
+                        if (count == 1) {
+                            return Optional.empty(); // no new information, so don't say anything
+                        }
+                    }
+                    return Optional.of(s);
+                }
+            }
+            if (count == 0 && !Cardinality.allowsZero(cardinality)) {
+                return Optional.of("The type does not allow an empty sequence");
+            } else if (count > 1 && !Cardinality.allowsMany(cardinality)) {
+                return Optional.of("The type does not allow a sequence of more than one item");
+            }
+            return Optional.empty();
+        } catch (XPathException e) {
+            return Optional.empty();
+        }
+    }
+
     /**
      * Return a string representation of this SequenceType
      *
@@ -492,19 +542,18 @@ public final class SequenceType {
         if (cardinality == StaticProperty.ALLOWS_ZERO) {
             return "empty-sequence()";
         } else {
-            return primaryType.toString() + Cardinality.getOccurrenceIndicator(cardinality);
+            return primaryType + Cardinality.getOccurrenceIndicator(cardinality);
         }
     }
 
     /**
-     * Return a string representation of this SequenceType suitable for use in export (SEF) files.
-     * The difference from the toString() method is that the representation will not contain
-     * references to anonymous types.
-     * @return the string representation as an instance of the XPath
-     * SequenceType construct, using only names that will be in-scope if the target environment
-     * imports the same schema as the source environment
+     * Return a string representation of this SequenceType suitable for use in stylesheet
+     * export files. This differs from the result of toString() in that it will not contain
+     * any references to anonymous types. Note that it may also use the Saxon extended syntax
+     * for union types and tuple types.
+     *
+     * @return the string representation as an instance of the XPath SequenceType construct
      */
-
     public String toExportString() {
         if (cardinality == StaticProperty.ALLOWS_ZERO) {
             return "empty-sequence()";
@@ -513,6 +562,15 @@ public final class SequenceType {
         }
     }
 
+    public String toExportString2() {
+        String result = null;
+//#if EE==true
+        MapItem map = ExperimentalXPathCompiler.makeMap(this);
+        result= ExperimentalSequenceTypeMarshaller.unmarshal(map).getStringValue();
+//#endif
+        return result;
+    }
+
     /**
      * Returns a hash code value for the object.
      */
